﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MyscreenSaver
{
    public partial class MainForm : Form
    {
        //ピクチャボックスの移動範囲を格納する変数
        private Rectangle VisibleRect;
        //ランダムな数値を発生させる変数
        private Random rand;

        #region Preview API's

        [DllImport("user32.dll")]
        static extern IntPtr SetParent(IntPtr hWndChild, IntPtr hWndNewParent);

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, IntPtr dwNewLong);

        [DllImport("user32.dll", SetLastError = true)]
        static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport("user32.dll")]
        static extern bool GetClientRect(IntPtr hWnd, out Rectangle lpRect);

        #endregion

        bool IsPreviewMode = false;

        #region Constructors

        public MainForm()
        {
            InitializeComponent();
        }

        // このコンストラクタは、フォームを全画面で表示する
        // ノーマルモードで使用される
        public MainForm(Rectangle Bounds)
        {
            InitializeComponent();

            this.Bounds = Bounds;
            //フォームを最前面に設定
            TopMost = true;
            // マウスカーソルを非表示にする
            Cursor.Hide();
        }

        // このコンストラクタは、スクリーンセーバーの選択ダイアログボックスの小窓で使用される。
        // プレビューモードで使用される (/p)
        public MainForm(IntPtr PreviewHandle)
        {
            InitializeComponent();

            // このウィンドの親ウィンドウにプレビューウィンドを設定
            SetParent(this.Handle, PreviewHandle);

            // この子ウィンドウをスクリーンセーバーの選択ダイアログボックスが閉じられたとき
            SetWindowLong(this.Handle, -16, new IntPtr(GetWindowLong(this.Handle, -16) | 0x40000000));

            //ウィンドウサイズを親ウィンドウのサイズに設定する
            Rectangle ParentRect;
            GetClientRect(PreviewHandle, out ParentRect);
            this.Size = ParentRect.Size;

            // ピクチャボックスのサイズを親ウィンドウの1/4にする。
            pictureBox1.Width = ParentRect.Width / 4; // 追加
            pictureBox1.Height = ParentRect.Height / 4; //　追加

            // ロケーションを(0, 0)に設定
            this.Location = new Point(0, 0);

            IsPreviewMode = true;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            //ピクチャボックスの移動範囲を変数に代入
            VisibleRect = new Rectangle(
                0, 0, Bounds.Width - pictureBox1.Width,
                Bounds.Height - pictureBox1.Height);
            //ピクチャボックスの位置をランダムに設定
            rand = new Random();
            pictureBox1.Location =
                new Point(rand.Next(VisibleRect.Width)
                , rand.Next(VisibleRect.Height));
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            //timer1で設定した時間がきたらピクチャボックスの位置を
            //ランダムに変更
            pictureBox1.Location =
                new Point(rand.Next(VisibleRect.Width)
                , rand.Next(VisibleRect.Height));
        }
        #endregion

        #region User Input

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            System.Environment.Exit(0);
        }

        private void MainForm_Click(object sender, EventArgs e)
        {
            //** レビューを実行していないとき
            if (!IsPreviewMode) // プレビュー画面用に無効にする
            {
                System.Environment.Exit(0);
            }
        }

        // XとY int.MaxValueのとOriginalLoctionを始める
        // カーソルがその位置にすることは不可能なので。
        // この変数がまだ設定されている場合
        Point OriginalLocation = new Point(int.MaxValue, int.MaxValue);

        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            //** プレビューを行っていない場合は、このif文を取り出す
            if (!IsPreviewMode) // プレビューのとき終了関数を無効にする
            {
                //originallocationが設定されているかどうかを確認
                if (OriginalLocation.X == int.MaxValue &
                    OriginalLocation.Y == int.MaxValue)
                {
                    OriginalLocation = e.Location;
                }
                // マウスが20 pixels 以上動いたかどうかを監視
                // 動いた場合はアプリケーションを終了
                if (Math.Abs(e.X - OriginalLocation.X) > 20 |
                    Math.Abs(e.Y - OriginalLocation.Y) > 20)
                {
                    System.Environment.Exit(0);
                }
            }
        }
        #endregion

 

       
    }
}
